@@ -11,6 +11,7 @@ use Dancer::RPCPlugin::DispatchFromConfig;
 use Dancer::RPCPlugin::DispatchFromPod;
 use Dancer::RPCPlugin::DispatchItem;
 use Dancer::RPCPlugin::DispatchMethodList;
+use Dancer::RPCPlugin::FlattenData;
 
 my %dispatch_builder_map = (
     pod    => \&build_dispatcher_from_pod,
@@ -30,13 +31,6 @@ register jsonrpc => sub {
             $publisher = $_;
         }
     }
-
-    my $code_wrapper = $arguments->{code_wrapper} // sub {
-        my $code = shift;
-        my $pkg  = shift;
-        $code->(@_);
-    };
-    my $callback = $arguments->{callback};
     my $dispatcher = $publisher->($arguments->{arguments}, $endpoint);
 
     my $lister = Dancer::RPCPlugin::DispatchMethodList->new();
@@ -46,17 +40,29 @@ register jsonrpc => sub {
         methods  => [ sort keys %{ $dispatcher } ],
     );
 
+    my $code_wrapper = $arguments->{code_wrapper}
+        ? $arguments->{code_wrapper}
+        : sub {
+            my $code = shift;
+            my $pkg  = shift;
+            $code->(@_);
+        };
+    my $callback = $arguments->{callback};
+
+    debug("Starting jsonrpc-handler build: ", $lister);
     my $handle_call = sub {
         if (request->content_type ne 'application/json') {
             pass();
         }
+        debug("[handle_jsonrpc_request] Processing: ", request->body);
 
         my @requests = unjson(request->body);
 
+        content_type 'application/json';
         my @responses;
         for my $request (@requests) {
             my $method_name = $request->{method};
-            debug("[handle_jsonrpc_call] $method_name ", $request);
+            debug("[handle_jsonrpc_call($method_name)] $method_name ", $request);
 
             if (!exists $dispatcher->{$method_name}) {
                 push(
@@ -92,18 +98,25 @@ register jsonrpc => sub {
                 );
                 next;
             }
-            if (!$continue->success) {
-                push(
-                    @responses,
-                    jsonrpc_error_response(
+            if (!blessed($continue) || !$continue->isa('Dancer::RPCPlugin::CallbackResult')) {
+                push @responses, jsonrpc_error_response(
+                    $request->{id},
+                    {
+                        code    => -32603,
+                        message => "Internal error: 'callback_result' wrong class " . blessed($continue),
+                    }
+                );
+                next;
+            }
+            elsif (blessed($continue) && !$continue->success) {
+                    push @responses, jsonrpc_error_response(
                         $request->{id},
                         {
                             code    => $continue->error_code,
                             message => $continue->error_message,
                         }
-                    )
-                );
-                next;
+                    );
+                    next;
             }
 
             my Dancer::RPCPlugin::DispatchItem $di = $dispatcher->{$method_name};
@@ -113,18 +126,28 @@ register jsonrpc => sub {
             my $result = eval {
                 $code_wrapper->($handler, $package, $method_name, @method_args);
             };
+            my $error = $@;
 
-            debug("[handeled_jsonrpc_call] ", $result);
-            if (my $error = $@) {
-                $result = {
-                    error => {
-                        code => 500,
+            debug("[handeled_jsonrpc_call($method_name)] ", $result);
+            if ($error) {
+                push @responses, jsonrpc_error_response(
+                    $request->{id},
+                    {
+                        code    => 500,
                         message => $error,
                     }
-                };
+                );
+                next;
             }
             if (blessed($result) && $result->can('as_jsonrpc_error')) {
-                $result = $result->as_jsonrpc_error;
+                push @responses, jsonrpc_error_response(
+                    $request->{id},
+                    $result->as_jsonrpc_error->{error}
+                );
+                next
+            }
+            elsif (blessed($result)) {
+                $result = flatten_data($result);
             }
             push @responses, jsonrpc_response($request->{id}, $result);
         }
@@ -135,13 +158,13 @@ register jsonrpc => sub {
             $response = to_json($responses[0]);
         }
         else {
-            $response = to_json(\@responses);
+            $response = to_json([grep {defined($_->{id})} @responses]);
         }
 
-        content_type 'application/json';
         return $response;
     };
 
+    debug("setting route (jsonrpc): $endpoint ", $lister);
     post $endpoint, $handle_call;
 };
 
@@ -162,9 +185,6 @@ sub unjson {
 sub jsonrpc_response {
     my ($id, $data) = @_;
 
-    if (ref($data) eq 'HASH' && exists $data->{error}) {
-        return jsonrpc_error_response($id => $data->{error});
-    }
     return {
         jsonrpc => '2.0',
         id      => $id,
@@ -201,6 +221,7 @@ sub build_dispatcher_from_config {
         endpoint => $endpoint,
     );
 }
+
 register_plugin;
 1;
 
